Struct rustsat::instances::Cnf

source ·
pub struct Cnf { /* private fields */ }
Simple type representing a CNF formula. Other than SatInstance<VM>, this type only supports clauses and does have an internal variable manager.

Implementations

source

impl Cnf

source

pub fn new() -> Cnf

Creates a new Cnf

source

pub fn with_capacity(capacity: usize) -> Cnf

Creates a new Cnf with a given capacity of clauses

source

pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError>

Tries to reserve memory for at least additional new clauses

source

pub fn shrink_to_fit(&mut self)

Shrinks the allocated memory of the Cnf to fit the number of clauses

source

pub fn capacity(&self) -> usize

Gets the capacity of the Cnf

source

pub fn add_clause(&mut self, clause: Clause)

Adds a clause to the CNF

source

pub fn add_unit(&mut self, unit: Lit)

Adds a unit clause to the CNF

source

pub fn add_binary(&mut self, lit1: Lit, lit2: Lit)

Adds a binary clause to the CNF

source

pub fn add_ternary(&mut self, lit1: Lit, lit2: Lit, lit3: Lit)

Adds a ternary clause to the CNF

source

pub fn is_empty(&self) -> bool

Checks if the CNF is empty

source

pub fn len(&self) -> usize

Returns the number of clauses in the instance

source

pub fn add_lit_impl_lit(&mut self, a: Lit, b: Lit)

source

pub fn add_lit_impl_clause(&mut self, a: Lit, b: &[Lit])

source

pub fn add_lit_impl_cube(&mut self, a: Lit, b: &[Lit])

Adds an implication of form a -> (b1 & b2 & … & bm)

source

pub fn add_cube_impl_lit(&mut self, a: &[Lit], b: Lit)

source

pub fn add_clause_impl_lit(&mut self, a: &[Lit], b: Lit)

Adds an implication of form (a1 | a2 | … | an) -> b

source

pub fn add_cube_impl_clause(&mut self, a: &[Lit], b: &[Lit])

source

pub fn add_clause_impl_clause(&mut self, a: &[Lit], b: &[Lit])

Adds an implication of form (a1 | a2 | … | an) -> (b1 | b2 | … | bm)

source

pub fn add_clause_impl_cube(&mut self, a: &[Lit], b: &[Lit])

Adds an implication of form (a1 | a2 | … | an) -> (b1 & b2 & … & bm)

source

pub fn add_cube_impl_cube(&mut self, a: &[Lit], b: &[Lit])

Adds an implication of form (a1 & a2 & … & an) -> (b1 & b2 & … & bm)

source

pub fn join(self, other: Cnf) -> Cnf

Joins the current CNF with another one. Like Cnf::extend but consumes the object and returns a new object.

source

pub fn iter(&self) -> Iter<'_, Clause>

Returns an iterator over references to the clauses

source

pub fn iter_mut(&mut self) -> IterMut<'_, Clause>

Returns an iterator over mutable references to the clauses

source

pub fn normalize(self) -> Self

Normalizes the CNF. This includes normalizing and sorting the clauses, removing duplicates and tautologies. Comparing two normalized CNFs is equal to comparing sets of sets of literals.

source

pub fn sanitize(self) -> Self

Sanitizes the CNF by removing tautologies, removing redundant literals, etc.