Interrupt

Trait Interrupt 

pub trait Interrupt {
    type Interrupter: InterruptSolver + Send + 'static;

    // Required method
    fn interrupter(&mut self) -> Self::Interrupter;
}
Trait for all solvers that can be asynchronously interrupt.

Required Associated Types

type Interrupter: InterruptSolver + Send + 'static

The interrupter of the solver

Required Methods

fn interrupter(&mut self) -> Self::Interrupter

Gets a thread safe interrupter object that can be used to terminate the solver

Implementors