pub trait Interrupt {
type Interrupter: InterruptSolver + Send + 'static;
// Required method
fn interrupter(&mut self) -> Self::Interrupter;
}Expand description
Trait for all solvers that can be asynchronously interrupt.
Required Associated Types
Sourcetype Interrupter: InterruptSolver + Send + 'static
type Interrupter: InterruptSolver + Send + 'static
The interrupter of the solver
Required Methods
Sourcefn interrupter(&mut self) -> Self::Interrupter
fn interrupter(&mut self) -> Self::Interrupter
Gets a thread safe interrupter object that can be used to terminate the solver