Common Types for SAT Solving
Common types used throughout the library to guarantee type safety.
Re-exports
pub use constraints::Clause;
Modules
- Constraint Types
Structs
- Type representing an assignment of variables.
- Type representing literals, possibly negated boolean variables.
- Type representing boolean variables in a SAT problem. Variables indexing in RustSAT starts from 0 and the maximum index is
(u32::MAX - 1) / 2. This is because literals are represented as a singleu32as well. The memory representation of variables isu32.
Enums
- Ternary value assigned to a literal or variable, including possible “don’t care”
- Errors related to types
Traits
- An iterator over clauses
- An iterator over integer-weighted literals
- An iterator over literals
- An iterator over weighted clauses
- An iterator over weighted literals
Type Aliases
- The hash map to use throughout the library
- The hash set to use throughout the library
- The hasher to use throught the library